@@ -27,8 +27,10 @@
 #include <thai/thwchar.h>
 #include <thai/thbrk.h>
 
-#ifndef HAVE_TH_BRK_FIND_BREAKS
-G_LOCK_DEFINE_STATIC (th_brk);
+G_LOCK_DEFINE_STATIC (thai_brk);
+
+#ifdef HAVE_TH_BRK_FIND_BREAKS
+static ThBrk *thai_brk = NULL;
 #endif
 
 /*
@@ -79,22 +81,24 @@ break_thai (const char          *text,
 
   /* find line break positions */
 
+  G_LOCK (thai_brk);
 #ifdef HAVE_TH_BRK_FIND_BREAKS
-  len = th_brk_find_breaks(NULL, tis_text, brk_pnts, cnt);
+  if (thai_brk == NULL)
+    thai_brk = th_brk_new(NULL);
+  len = th_brk_find_breaks(thai_brk, tis_text, brk_pnts, cnt);
 #else
-  G_LOCK (th_brk);
   len = th_brk (tis_text, brk_pnts, cnt);
-  G_UNLOCK (th_brk);
 #endif
+  G_UNLOCK (thai_brk);
 
   for (cnt = 0; cnt < len; cnt++)
-    if (attrs[brk_pnts[cnt]].is_line_break)
+    if (attrs[brk_pnts[cnt]].is_char_break)
     {
-      /* Only allow additional char breaks if line-breaking is NOT
-       * prohibited. (The alternative would be to set is_line_break to
+      /* Only allow additional line breaks if line-breaking is NOT
+       * prohibited. (The alternative would be to set is_char_break to
        * TRUE as well.  NOT setting it will break invariants that any
        * line break opportunity is also a char break opportunity. */
-      attrs[brk_pnts[cnt]].is_char_break = TRUE;
+      attrs[brk_pnts[cnt]].is_line_break = TRUE;
       attrs[brk_pnts[cnt]].is_word_start = TRUE;
       attrs[brk_pnts[cnt]].is_word_end = TRUE;
     }
