@@ -38,7 +38,7 @@
  * A `PangoContext` stores global information used to control the
  * itemization process.
  *
- * The information stored by `PangoContext includes the fontmap used
+ * The information stored by `PangoContext` includes the fontmap used
  * to look up fonts, and default values such as the default language,
  * default gravity, or default font.
  *
@@ -169,7 +169,7 @@ update_resolved_gravity (PangoContext *context)
 /**
  * pango_context_set_matrix:
  * @context: a `PangoContext`
- * @matrix: (allow-none): a `PangoMatrix`, or %NULL to unset any existing
+ * @matrix: (nullable): a `PangoMatrix`, or %NULL to unset any existing
  * matrix. (No matrix set is the same as setting the identity matrix.)
  *
  * Sets the transformation matrix that will be applied when rendering
@@ -403,7 +403,7 @@ pango_context_get_font_description (PangoContext *context)
  * Sets the global language tag for the context.
  *
  * The default language for the locale of the running process
- * can be found using [type_func@Pango.Language.get_default].
+ * can be found using [func@Pango.Language.get_default].
  */
 void
 pango_context_set_language (PangoContext  *context,
@@ -534,7 +534,7 @@ pango_context_get_base_gravity (PangoContext *context)
  *
  * This is similar to [method@Pango.Context.get_base_gravity],
  * except for when the base gravity is %PANGO_GRAVITY_AUTO for
- * which [type_func@Pango.Gravity.get_for_matrix] is used to return the
+ * which [func@Pango.Gravity.get_for_matrix] is used to return the
  * gravity from the current context matrix.
  *
  * Return value: the resolved gravity for the context.
@@ -629,6 +629,7 @@ typedef struct {
 
 typedef struct {
   PangoFont *font;
+  int position; /* position of the font in the fontset */
 } FontElement;
 
 static void
@@ -677,7 +678,8 @@ retry:
 static gboolean
 font_cache_get (FontCache   *cache,
                 gunichar     wc,
-                PangoFont  **font)
+                PangoFont  **font,
+                int         *position)
 {
   FontElement *element;
 
@@ -685,7 +687,7 @@ font_cache_get (FontCache   *cache,
   if (element)
     {
       *font = element->font;
-
+      *position = element->position;
       return TRUE;
     }
   else
@@ -693,12 +695,14 @@ font_cache_get (FontCache   *cache,
 }
 
 static void
-font_cache_insert (FontCache   *cache,
-                   gunichar           wc,
-                   PangoFont         *font)
+font_cache_insert (FontCache *cache,
+                   gunichar   wc,
+                   PangoFont *font,
+                   int        position)
 {
   FontElement *element = g_slice_new (FontElement);
   element->font = font ? g_object_ref (font) : NULL;
+  element->position = position;
 
   g_hash_table_insert (cache->hash, GUINT_TO_POINTER (wc), element);
 }
@@ -779,6 +783,9 @@ struct _ItemizeState
   FontCache *cache;
   PangoFont *base_font;
   gboolean enable_fallback;
+
+  const char *first_space; /* first of a sequence of spaces we've seen */
+  int font_position; /* position of the current font in the fontset */
 };
 
 static void
@@ -1024,6 +1031,11 @@ itemize_state_init (ItemizeState               *state,
   state->embedding_end = text + start_index;
   update_embedding_end (state);
 
+  state->gravity = PANGO_GRAVITY_AUTO;
+  state->centered_baseline = PANGO_GRAVITY_IS_VERTICAL (state->context->resolved_gravity);
+  state->gravity_hint = state->context->gravity_hint;
+  state->resolved_gravity = PANGO_GRAVITY_AUTO;
+
   /* Initialize the attribute iterator
    */
   if (cached_iter)
@@ -1081,15 +1093,12 @@ itemize_state_init (ItemizeState               *state,
   else
     state->font_desc_gravity = PANGO_GRAVITY_AUTO;
 
-  state->gravity = PANGO_GRAVITY_AUTO;
-  state->centered_baseline = PANGO_GRAVITY_IS_VERTICAL (state->context->resolved_gravity);
-  state->gravity_hint = state->context->gravity_hint;
-  state->resolved_gravity = PANGO_GRAVITY_AUTO;
   state->derived_lang = NULL;
   state->current_fonts = NULL;
   state->cache = NULL;
   state->base_font = NULL;
-
+  state->first_space = NULL;
+  state->font_position = 0xffff;
 }
 
 static gboolean
@@ -1170,18 +1179,33 @@ itemize_state_fill_font (ItemizeState *state,
 static void
 itemize_state_add_character (ItemizeState *state,
                              PangoFont    *font,
+                             int           font_position,
                              gboolean      force_break,
-                             const char   *pos)
+                             const char   *pos,
+                             gboolean      is_space)
 {
+  const char *first_space = state->first_space;
+  int n_spaces = 0;
+
+  if (is_space)
+    {
+      if (state->first_space == NULL)
+        state->first_space = pos;
+    }
+  else
+    state->first_space = NULL;
+
   if (state->item)
     {
       if (!state->item->analysis.font && font)
         {
           itemize_state_fill_font (state, font);
+          state->font_position = font_position;
         }
       else if (state->item->analysis.font && !font)
         {
           font = state->item->analysis.font;
+          font_position = state->font_position;
         }
 
       if (!force_break &&
@@ -1191,17 +1215,33 @@ itemize_state_add_character (ItemizeState *state,
           return;
         }
 
+      /* Font is changing, we are about to end the current item.
+       * If it ended in a sequence of spaces (but wasn't only spaces),
+       * check if we should move those spaces to the new item (since
+       * the font is less "fallback".
+       *
+       * See https://gitlab.gnome.org/GNOME/pango/-/issues/249
+       */
+      if (state->text + state->item->offset < first_space &&
+          font_position < state->font_position)
+        {
+          n_spaces = g_utf8_strlen (first_space, pos - first_space);
+          state->item->num_chars -= n_spaces;
+          pos = first_space;
+        }
+
       state->item->length = (pos - state->text) - state->item->offset;
     }
 
   state->item = pango_item_new ();
   state->item->offset = pos - state->text;
   state->item->length = 0;
-  state->item->num_chars = 1;
+  state->item->num_chars = n_spaces + 1;
 
   if (font)
     g_object_ref (font);
   state->item->analysis.font = font;
+  state->font_position = font_position;
 
   state->item->analysis.level = state->embedding;
   state->item->analysis.gravity = state->resolved_gravity;
@@ -1259,6 +1299,7 @@ typedef struct {
   PangoLanguage *lang;
   gunichar wc;
   PangoFont *font;
+  int position;
 } GetFontInfo;
 
 static gboolean
@@ -1283,6 +1324,8 @@ get_font_foreach (PangoFontset *fontset,
       return TRUE;
     }
 
+  info->position++;
+
   return FALSE;
 }
 
@@ -1299,18 +1342,20 @@ get_base_font (ItemizeState *state)
 static gboolean
 get_font (ItemizeState  *state,
           gunichar       wc,
-          PangoFont    **font)
+          PangoFont    **font,
+          int           *position)
 {
   GetFontInfo info;
 
   /* We'd need a separate cache when fallback is disabled, but since lookup
    * with fallback disabled is faster anyways, we just skip caching */
-  if (state->enable_fallback && font_cache_get (state->cache, wc, font))
+  if (state->enable_fallback && font_cache_get (state->cache, wc, font, position))
     return TRUE;
 
   info.lang = state->derived_lang;
   info.wc = wc;
   info.font = NULL;
+  info.position = 0;
 
   if (state->enable_fallback)
     pango_fontset_foreach (state->current_fonts, get_font_foreach, &info);
@@ -1318,10 +1363,11 @@ get_font (ItemizeState  *state,
     get_font_foreach (NULL, get_base_font (state), &info);
 
   *font = info.font;
+  *position = info.position;
 
   /* skip caching if fallback disabled (see above) */
   if (state->enable_fallback)
-    font_cache_insert (state->cache, wc, *font);
+    font_cache_insert (state->cache, wc, *font, *position);
 
   return TRUE;
 }
@@ -1436,6 +1482,7 @@ itemize_state_process_run (ItemizeState *state)
 {
   const char *p;
   gboolean last_was_forced_break = FALSE;
+  gboolean is_space;
 
   /* Only one character has type G_UNICODE_LINE_SEPARATOR in Unicode 4.0;
    * update this if that changes. */
@@ -1453,6 +1500,7 @@ itemize_state_process_run (ItemizeState *state)
       gunichar wc = g_utf8_get_char (p);
       gboolean is_forced_break = (wc == '\t' || wc == LINE_SEPARATOR);
       PangoFont *font;
+      int font_position;
       GUnicodeType type;
 
       /* We don't want space characters to affect font selection; in general,
@@ -1466,6 +1514,10 @@ itemize_state_process_run (ItemizeState *state)
        * See bug #781123.
        *
        * Finally, don't change fonts for line or paragraph separators.
+       *
+       * Note that we want spaces to use the 'better' font, comparing
+       * the font that is used before and after the space. This is handled
+       * in itemize_state_add_character().
        */
       type = g_unichar_type (wc);
       if (G_UNLIKELY (type == G_UNICODE_CONTROL ||
@@ -1478,15 +1530,19 @@ itemize_state_process_run (ItemizeState *state)
                       (wc >= 0xe0100u && wc <= 0xe01efu)))
         {
           font = NULL;
+          font_position = 0xffff;
+          is_space = TRUE;
         }
       else
         {
-          get_font (state, wc, &font);
+          get_font (state, wc, &font, &font_position);
+          is_space = FALSE;
         }
 
-      itemize_state_add_character (state, font,
+      itemize_state_add_character (state, font, font_position,
                                    is_forced_break || last_was_forced_break,
-                                   p);
+                                   p,
+                                   is_space);
 
       last_was_forced_break = is_forced_break;
     }
@@ -1496,8 +1552,9 @@ itemize_state_process_run (ItemizeState *state)
   if (!state->item->analysis.font)
     {
       PangoFont *font;
+      int position;
 
-      if (G_UNLIKELY (!get_font (state, ' ', &font)))
+      if (G_UNLIKELY (!get_font (state, ' ', &font, &position)))
         {
           /* If no font was found, warn once per fontmap/script pair */
           PangoFontMap *fontmap = state->context->font_map;
@@ -1549,7 +1606,7 @@ itemize_state_finish (ItemizeState *state)
  * @length: the number of bytes (not characters) to process
  *   after @start_index. This must be >= 0.
  * @attrs: the set of attributes that apply to @text.
- * @cached_iter: (allow-none): Cached attribute iterator, or %NULL
+ * @cached_iter: (nullable): Cached attribute iterator
  *
  * Like `pango_itemize()`, but with an explicitly specified base direction.
  *
@@ -1627,7 +1684,7 @@ itemize_with_font (PangoContext               *context,
  * @length: the number of bytes (not characters) to process
  *   after @start_index. This must be >= 0.
  * @attrs: the set of attributes that apply to @text.
- * @cached_iter: (allow-none): Cached attribute iterator, or %NULL
+ * @cached_iter: (nullable): Cached attribute iterator
  *
  * Breaks a piece of text into segments with consistent directional
  * level and font.
@@ -1749,12 +1806,12 @@ update_metrics_from_items (PangoFontMetrics *metrics,
 /**
  * pango_context_get_metrics:
  * @context: a `PangoContext`
- * @desc: (allow-none): a `PangoFontDescription` structure. %NULL means that the
+ * @desc: (nullable): a `PangoFontDescription` structure. %NULL means that the
  *   font description from the context will be used.
- * @language: (allow-none): language tag used to determine which script to get
+ * @language: (nullable): language tag used to determine which script to get
  *   the metrics for. %NULL means that the language tag from the context
  *   will be used. If no language tag is set on the context, metrics
- *   for the default language (as determined by [type_func@Pango.Language.get_default]
+ *   for the default language (as determined by [func@Pango.Language.get_default]
  *   will be returned.
  *
  * Get overall metric information for a particular font description.
