@@ -294,12 +294,16 @@ markup_data_close_tag (MarkupData *md)
 
       if (ot->has_base_font_size)
 	{
-	  /* Create a font using the absolute point size
-	   * as the base size to be scaled from
-	   */
-	  a = pango_attr_size_new (scale_factor (ot->scale_level,
-						 1.0) *
-				   ot->base_font_size);
+          /* Create a font using the absolute point size as the base size
+           * to be scaled from.
+           * We need to use a local variable to ensure that the compiler won't
+           * implicitly cast it to integer while the result is kept in registers,
+           * leading to a wrong approximation in i386 (with 387 FPU)
+           */
+          volatile double size;
+
+          size = scale_factor (ot->scale_level, 1.0) * ot->base_font_size;
+          a = pango_attr_size_new (size);
 	}
       else
 	{
@@ -584,17 +588,10 @@ pango_markup_parser_new_internal (char       accel_marker,
 					0, md,
                                         (GDestroyNotify)destroy_markup_data);
 
-  if (!g_markup_parse_context_parse (context,
-                                     "<markup>",
-                                     -1,
-                                     error))
-    goto error;
+  if (!g_markup_parse_context_parse (context, "<markup>", -1, error))
+    g_clear_pointer (&context, g_markup_parse_context_free);
 
   return context;
-
- error:
-  g_markup_parse_context_free (context);
-  return NULL;
 }
 
 /**
@@ -602,10 +599,10 @@ pango_markup_parser_new_internal (char       accel_marker,
  * @markup_text: markup to parse (see the Pango Markup docs)
  * @length: length of @markup_text, or -1 if nul-terminated
  * @accel_marker: character that precedes an accelerator, or 0 for none
- * @attr_list: (out) (allow-none): address of return location for a `PangoAttrList`, or %NULL
- * @text: (out) (allow-none): address of return location for text with tags stripped, or %NULL
- * @accel_char: (out) (allow-none): address of return location for accelerator char, or %NULL
- * @error: address of return location for errors, or %NULL
+ * @attr_list: (out) (optional): address of return location for a `PangoAttrList`
+ * @text: (out) (optional): address of return location for text with tags stripped
+ * @accel_char: (out) (optional): address of return location for accelerator char
+ * @error: address of return location for errors
  *
  * Parses marked-up text to create a plain-text string and an attribute list.
  *
@@ -654,8 +651,6 @@ pango_parse_markup (const char                 *markup_text,
   context = pango_markup_parser_new_internal (accel_marker,
                                               error,
                                               (attr_list != NULL));
-  if (context == NULL)
-    goto out;
 
   if (!g_markup_parse_context_parse (context,
                                      markup_text,
@@ -711,25 +706,18 @@ pango_parse_markup (const char                 *markup_text,
  * Since: 1.31.0
  **/
 GMarkupParseContext *
-pango_markup_parser_new (gunichar               accel_marker)
+pango_markup_parser_new (gunichar accel_marker)
 {
-  GError *error = NULL;
-  GMarkupParseContext *context;
-  context = pango_markup_parser_new_internal (accel_marker, &error, TRUE);
-
-  if (context == NULL)
-    g_critical ("Had error when making markup parser: %s\n", error->message);
-
-  return context;
+  return pango_markup_parser_new_internal (accel_marker, NULL, TRUE);
 }
 
 /**
  * pango_markup_parser_finish:
  * @context: A valid parse context that was returned from [func@markup_parser_new]
- * @attr_list: (out) (allow-none): address of return location for a `PangoAttrList`, or %NULL
- * @text: (out) (allow-none): address of return location for text with tags stripped, or %NULL
- * @accel_char: (out) (allow-none): address of return location for accelerator char, or %NULL
- * @error: address of return location for errors, or %NULL
+ * @attr_list: (out) (optional): address of return location for a `PangoAttrList`
+ * @text: (out) (optional): address of return location for text with tags stripped
+ * @accel_char: (out) (optional): address of return location for accelerator char
+ * @error: address of return location for errors
  *
  * Finishes parsing markup.
  *
@@ -1245,11 +1233,16 @@ span_parse_func     (MarkupData            *md G_GNUC_UNUSED,
 	CHECK_ATTRIBUTE (underline);
 	CHECK_ATTRIBUTE (underline_color);
 	break;
-      default:
+      case 'r':
 	CHECK_ATTRIBUTE (rise);
+        break;
+      case 'v':
 	CHECK_ATTRIBUTE (variant);
+        break;
+      case 'w':
 	CHECK_ATTRIBUTE (weight);
 	break;
+      default:;
       }
 
       if (!found)
@@ -1657,6 +1650,7 @@ markup_parse_func (MarkupData            *md G_GNUC_UNUSED,
 		   GError               **error G_GNUC_UNUSED)
 {
   /* We don't do anything with this tag at the moment. */
+  CHECK_NO_ATTRS("markup");
 
   return TRUE;
 }
