@@ -32,13 +32,11 @@ G_DEFINE_BOXED_TYPE (PangoMatrix, pango_matrix,
 
 /**
  * pango_matrix_copy:
- * @matrix: (nullable): a `PangoMatrix`, may be %NULL
+ * @matrix: (nullable): a `PangoMatrix`
  *
  * Copies a `PangoMatrix`.
  *
- * Return value: (nullable): the newly allocated `PangoMatrix`, which
- *   should be freed with [method@Pango.Matrix.free], or %NULL if
- *   @matrix was %NULL.
+ * Return value: (nullable): the newly allocated `PangoMatrix`
  *
  * Since: 1.6
  */
@@ -188,7 +186,7 @@ pango_matrix_concat (PangoMatrix       *matrix,
 
 /**
  * pango_matrix_get_font_scale_factor:
- * @matrix: (allow-none): a `PangoMatrix`, may be %NULL
+ * @matrix: (nullable): a `PangoMatrix`, may be %NULL
  *
  * Returns the scale factor of a matrix on the height of the font.
  *
@@ -211,9 +209,9 @@ pango_matrix_get_font_scale_factor (const PangoMatrix *matrix)
 
 /**
  * pango_matrix_get_font_scale_factors:
- * @matrix: (nullable): a `PangoMatrix`, or %NULL
- * @xscale: (out) (allow-none): output scale factor in the x direction, or %NULL
- * @yscale: (out) (allow-none): output scale factor perpendicular to the x direction, or %NULL
+ * @matrix: (nullable): a `PangoMatrix`
+ * @xscale: (out) (optional): output scale factor in the x direction
+ * @yscale: (out) (optional): output scale factor perpendicular to the x direction
  *
  * Calculates the scale factor of a matrix on the width and height of the font.
  *
@@ -266,7 +264,7 @@ pango_matrix_get_font_scale_factors (const PangoMatrix *matrix,
 
 /**
  * pango_matrix_transform_distance:
- * @matrix: (nullable): a `PangoMatrix`, or %NULL
+ * @matrix: (nullable): a `PangoMatrix`
  * @dx: (inout): in/out X component of a distance vector
  * @dy: (inout): in/out Y component of a distance vector
  *
@@ -307,7 +305,7 @@ pango_matrix_transform_distance (const PangoMatrix *matrix,
 
 /**
  * pango_matrix_transform_point:
- * @matrix: (nullable): a `PangoMatrix`, or %NULL
+ * @matrix: (nullable): a `PangoMatrix`
  * @x: (inout): in/out X position
  * @y: (inout): in/out Y position
  *
@@ -331,8 +329,8 @@ pango_matrix_transform_point (const PangoMatrix *matrix,
 
 /**
  * pango_matrix_transform_rectangle:
- * @matrix: (nullable): a `PangoMatrix`, or %NULL
- * @rect: (inout) (allow-none): in/out bounding box in Pango units, or %NULL
+ * @matrix: (nullable): a `PangoMatrix`
+ * @rect: (inout) (optional): in/out bounding box in Pango units
  *
  * First transforms @rect using @matrix, then calculates the bounding box
  * of the transformed rectangle.
@@ -411,8 +409,8 @@ pango_matrix_transform_rectangle (const PangoMatrix *matrix,
 
 /**
  * pango_matrix_transform_pixel_rectangle:
- * @matrix: (nullable): a `PangoMatrix`, or %NULL
- * @rect: (inout) (allow-none): in/out bounding box in device units, or %NULL
+ * @matrix: (nullable): a `PangoMatrix`
+ * @rect: (inout) (optional): in/out bounding box in device units
  *
  * First transforms the @rect using @matrix, then calculates the bounding box
  * of the transformed rectangle.
@@ -463,17 +461,18 @@ pango_matrix_transform_pixel_rectangle (const PangoMatrix *matrix,
   min_x = max_x = quad_x[0];
   min_y = max_y = quad_y[0];
 
-  for (i=1; i < 4; i++) {
+  for (i=1; i < 4; i++)
+    {
       if (quad_x[i] < min_x)
-	  min_x = quad_x[i];
+        min_x = quad_x[i];
       else if (quad_x[i] > max_x)
-	  max_x = quad_x[i];
+        max_x = quad_x[i];
 
       if (quad_y[i] < min_y)
-	  min_y = quad_y[i];
+        min_y = quad_y[i];
       else if (quad_y[i] > max_y)
-	  max_y = quad_y[i];
-  }
+        max_y = quad_y[i];
+    }
 
   rect->x      = floor (min_x);
   rect->y      = floor (min_y);
