@@ -100,13 +100,24 @@
  *   FcCharSetMerge().
  */
 
+typedef enum {
+  /* Initial state; Fontconfig is not initialized yet */
+  DEFAULT_CONFIG_NOT_INITIALIZED,
+
+  /* We have a thread doing Fontconfig initialization in the background */
+  DEFAULT_CONFIG_INITIALIZING,
+
+  /* FcInit() finished and its default configuration is loaded */
+  DEFAULT_CONFIG_INITIALIZED
+} DefaultConfig;
+
 /* We call FcInit in a thread and set fc_initialized
  * when done, and are protected by a mutex. The thread
  * signals the cond when FcInit is done.
  */
 static GMutex fc_init_mutex;
 static GCond fc_init_cond;
-static int fc_initialized = 0;
+static DefaultConfig fc_initialized = DEFAULT_CONFIG_NOT_INITIALIZED;
 
 
 typedef struct _PangoFcFontFaceData PangoFcFontFaceData;
@@ -743,8 +754,6 @@ pango_fc_font_key_get_variations (const PangoFcFontKey *key)
  */
 
 struct _PangoFcPatterns {
-  guint ref_count;
-
   PangoFcFontMap *fontmap;
 
   /* match and fontset are initialized in a thread,
@@ -883,11 +892,10 @@ pango_fc_patterns_new (FcPattern *pat, PangoFcFontMap *fontmap)
   if (pats)
     return pango_fc_patterns_ref (pats);
 
-  pats = g_slice_new0 (PangoFcPatterns);
+  pats = g_atomic_rc_box_new0 (PangoFcPatterns);
 
   pats->fontmap = fontmap;
 
-  pats->ref_count = 1;
   FcPatternReference (pat);
   pats->pattern = pat;
 
@@ -909,22 +917,13 @@ pango_fc_patterns_new (FcPattern *pat, PangoFcFontMap *fontmap)
 static PangoFcPatterns *
 pango_fc_patterns_ref (PangoFcPatterns *pats)
 {
-  g_return_val_if_fail (pats->ref_count > 0, NULL);
-
-  pats->ref_count++;
-
-  return pats;
+  return g_atomic_rc_box_acquire (pats);
 }
 
 static void
-pango_fc_patterns_unref (PangoFcPatterns *pats)
+free_patterns (gpointer data)
 {
-  g_return_if_fail (pats->ref_count > 0);
-
-  pats->ref_count--;
-
-  if (pats->ref_count)
-    return;
+  PangoFcPatterns *pats = data;
 
   /* Only remove from fontmap hash if we are in it.  This is not necessarily
    * the case after a cache_clear() call. */
@@ -944,8 +943,12 @@ pango_fc_patterns_unref (PangoFcPatterns *pats)
 
   g_cond_clear (&pats->cond);
   g_mutex_clear (&pats->mutex);
+}
 
-  g_slice_free (PangoFcPatterns, pats);
+static void
+pango_fc_patterns_unref (PangoFcPatterns *pats)
+{
+  g_atomic_rc_box_release_full (pats, free_patterns);
 }
 
 static FcPattern *
@@ -959,12 +962,20 @@ pango_fc_is_supported_font_format (FcPattern* pattern)
 {
   FcResult res;
   const char *fontformat;
+  const char *file;
+
+  /* Harfbuzz loads woff fonts, but we don't get any glyphs */
+  res = FcPatternGetString (pattern, FC_FILE, 0, (FcChar8 **)(void*)&file);
+  if (res == FcResultMatch &&
+      (g_str_has_suffix (file, ".woff") ||
+       g_str_has_suffix (file, ".woff2")))
+    return FALSE;
 
   res = FcPatternGetString (pattern, FC_FONTFORMAT, 0, (FcChar8 **)(void*)&fontformat);
   if (res != FcResultMatch)
     return FALSE;
 
-  /* harfbuzz supports only SFNT fonts. */
+  /* Harfbuzz supports only SFNT fonts. */
   /* FIXME: "CFF" is used for both CFF in OpenType and bare CFF files, but
    * HarfBuzz does not support the later and FontConfig does not seem
    * to have a way to tell them apart.
@@ -1350,7 +1361,7 @@ init_in_thread (gpointer task_data)
   pango_trace_mark (before, "FcInit", NULL);
 
   g_mutex_lock (&fc_init_mutex);
-  fc_initialized = 2;
+  fc_initialized = DEFAULT_CONFIG_INITIALIZED;
   g_cond_broadcast (&fc_init_cond);
   g_mutex_unlock (&fc_init_mutex);
 
@@ -1362,11 +1373,11 @@ start_init_in_thread (PangoFcFontMap *fcfontmap)
 {
   g_mutex_lock (&fc_init_mutex);
 
-  if (fc_initialized == 0)
+  if (fc_initialized == DEFAULT_CONFIG_NOT_INITIALIZED)
     {
       GThread *thread;
 
-      fc_initialized = 1;
+      fc_initialized = DEFAULT_CONFIG_INITIALIZING;
       thread = g_thread_new ("[pango] FcInit", init_in_thread, NULL);
       g_thread_unref (thread);
     }
@@ -1381,7 +1392,7 @@ wait_for_fc_init (void)
   gboolean waited = FALSE;
 
   g_mutex_lock (&fc_init_mutex);
-  while (fc_initialized < 2)
+  while (fc_initialized < DEFAULT_CONFIG_INITIALIZED)
     {
       waited = TRUE;
       g_cond_wait (&fc_init_cond, &fc_init_mutex);
@@ -2323,9 +2334,6 @@ pango_fc_font_map_set_config (PangoFcFontMap *fcfontmap,
 
   if (oldconfig)
     FcConfigDestroy (oldconfig);
-
-  /* No need to wait anymore */
-  fc_initialized = 2;
 }
 
 /**
@@ -2337,7 +2345,8 @@ pango_fc_font_map_set_config (PangoFcFontMap *fcfontmap,
  * See also: [method@PangoFc.FontMap.set_config].
  *
  * Returns: (nullable): the `FcConfig` object attached to
- *   @fcfontmap, which might be %NULL.
+ *   @fcfontmap, which might be %NULL. The return value is
+ *   owned by Pango and should not be freed.
  *
  * Since: 1.38
  */
@@ -2529,9 +2538,9 @@ _pango_fc_font_map_fc_to_languages (FcLangSet *langset)
   FcStrSet *strset;
   FcStrList *list;
   FcChar8 *s;
-  GArray *langs;
+  GPtrArray *langs;
 
-  langs = g_array_new (TRUE, FALSE, sizeof (PangoLanguage *));
+  langs = g_ptr_array_new ();
 
   strset = FcLangSetGetLangs (langset);
   list = FcStrListCreate (strset);
@@ -2540,13 +2549,15 @@ _pango_fc_font_map_fc_to_languages (FcLangSet *langset)
   while ((s = FcStrListNext (list)))
     {
       PangoLanguage *l = pango_language_from_string ((const char *)s);
-      g_array_append_val (langs, l);
+      g_ptr_array_add (langs, l);
     }
 
   FcStrListDone (list);
   FcStrSetDestroy (strset);
 
-  return (PangoLanguage **) g_array_free (langs, FALSE);
+  g_ptr_array_add (langs, NULL);
+
+  return (PangoLanguage **) g_ptr_array_free (langs, FALSE);
 }
 
 PangoLanguage **
@@ -2778,6 +2789,7 @@ pango_fc_font_description_from_pattern (FcPattern *pattern, gboolean include_siz
     {
       FcMatrix *fc_matrix;
       double scale_factor = 1;
+      volatile double scaled_size;
 
       if (FcPatternGetMatrix (pattern, FC_MATRIX, 0, &fc_matrix) == FcResultMatch)
         {
@@ -2791,7 +2803,12 @@ pango_fc_font_description_from_pattern (FcPattern *pattern, gboolean include_siz
           scale_factor = pango_matrix_get_font_scale_factor (&mat);
         }
 
-      pango_font_description_set_size (desc, scale_factor * size * PANGO_SCALE);
+      /* We need to use a local variable to ensure that the compiler won't
+       * implicitly cast it to integer while the result is kept in registers,
+       * leading to a wrong approximation in i386 (with 387 FPU)
+       */
+      scaled_size = scale_factor * size * PANGO_SCALE;
+      pango_font_description_set_size (desc, scaled_size);
     }
 
   /* gravity is a bit different.  we don't want to set it if it was not set on
@@ -3380,9 +3397,6 @@ pango_fc_font_map_get_hb_face (PangoFcFontMap *fcfontmap,
     {
       hb_blob_t *blob;
 
-      if (!hb_version_atleast (2, 0, 0))
-        g_error ("Harfbuzz version too old (%s)\n", hb_version_string ());
-
       blob = hb_blob_create_from_file (data->filename);
       data->hb_face = hb_face_create (blob, data->id);
       hb_blob_destroy (blob);
