@@ -619,7 +619,6 @@ lookup_aliases (GHashTable   *aliases_ht,
 {
   struct PangoAlias alias_key;
   struct PangoAlias *alias;
-  static gsize aliases_inited = 0;
 
   alias_key.alias = g_ascii_strdown (fontname, -1);
   alias = g_hash_table_lookup (aliases_ht, &alias_key);
@@ -678,7 +677,7 @@ create_standard_family (PangoWin32FontMap *win32fontmap,
 
               new_face->has_cmap = old_face->has_cmap;
               new_face->cmap_format = old_face->cmap_format;
-              new_face->cmap = old_face->cmap;
+              new_face->cmap = _pango_win32_copy_cmap (old_face->cmap, old_face->cmap_format);
 
               new_face->cached_fonts = NULL;
 
@@ -1038,19 +1037,35 @@ pango_win32_font_map_load_font (PangoFontMap               *fontmap,
       win32family = g_hash_table_lookup (win32fontmap->families, families[i]);
       if (win32family)
         {
+          PangoFontDescription *new_desc;
+          PangoFontDescription *best_desc = NULL;
+
           PING (("got win32family"));
           tmp_list = win32family->faces;
+
           while (tmp_list)
             {
               PangoWin32Face *face = tmp_list->data;
+              new_desc = pango_font_face_describe (PANGO_FONT_FACE (face));
+              pango_font_description_set_gravity (new_desc,
+                                                  pango_font_description_get_gravity (description));
 
               if (pango_font_description_better_match (description,
-                                                       best_match ? best_match->description : NULL,
-                                                       face->description))
-                best_match = face;
+                                                       best_desc,
+                                                       new_desc))
+                {
+                  pango_font_description_free (best_desc);
+                  best_desc = new_desc;
+                  best_match = face;
+                }
+              else
+                pango_font_description_free (new_desc);
 
               tmp_list = tmp_list->next;
             }
+
+          if (best_desc != NULL)
+            pango_font_description_free (best_desc);
         }
     }
   
@@ -1761,7 +1776,7 @@ pango_win32_face_finalize (GObject *object)
 
   g_free (win32face->face_name);
 
-  //g_free (win32face->cmap); // Err, cmap does not have lifecycle management currently :(
+  g_free (win32face->cmap);
 
   g_slist_free (win32face->cached_fonts);
 //  g_slist_free_full (win32face->cached_fonts, g_object_unref); // This doesn't work.
