@@ -54,12 +54,10 @@
 
 #include "pangowin32.h"
 #include "pango-font-private.h"
-#include "pango-fontset.h"
+#include "pango-fontset-private.h"
 #include "pango-fontmap-private.h"
 
-G_BEGIN_DECLS
-
-#define PANGO_TYPE_WIN32_FONT_MAP             (pango_win32_font_map_get_type ())
+#define PANGO_TYPE_WIN32_FONT_MAP             (_pango_win32_font_map_get_type ())
 #define PANGO_WIN32_FONT_MAP(object)          (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_TYPE_WIN32_FONT_MAP, PangoWin32FontMap))
 #define PANGO_WIN32_IS_FONT_MAP(object)       (G_TYPE_CHECK_INSTANCE_TYPE ((object), PANGO_TYPE_WIN32_FONT_MAP))
 #define PANGO_WIN32_FONT_MAP_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), PANGO_TYPE_WIN32_FONT_MAP, PangoWin32FontMapClass))
@@ -75,7 +73,6 @@ G_BEGIN_DECLS
 
 typedef struct _PangoWin32FontMap      PangoWin32FontMap;
 typedef struct _PangoWin32FontMapClass PangoWin32FontMapClass;
-typedef struct _PangoWin32DWriteItems  PangoWin32DWriteItems;
 typedef struct _PangoWin32Font         PangoWin32Font;
 typedef struct _PangoWin32FontClass    PangoWin32FontClass;
 typedef struct _PangoWin32Face         PangoWin32Face;
@@ -83,15 +80,12 @@ typedef PangoFontFaceClass             PangoWin32FaceClass;
 typedef struct _PangoWin32GlyphInfo    PangoWin32GlyphInfo;
 typedef struct _PangoWin32MetricsInfo  PangoWin32MetricsInfo;
 
-typedef struct _PangoWin32DWriteFontSetBuilder PangoWin32DWriteFontSetBuilder;
-
 struct _PangoWin32FontMap
 {
   PangoFontMap parent_instance;
 
   PangoWin32FontCache *font_cache;
   GQueue *freed_fonts;
-  guint serial;
 
   /* Map Pango family names to PangoWin32Family structs */
   GHashTable *families;
@@ -102,18 +96,13 @@ struct _PangoWin32FontMap
    */
   GHashTable *fonts;
 
-  /* Map LOGFONTWs to IDWriteFonts corresponding to actual fonts
-   * installed, if applicable.
-   */
-  GHashTable *dwrite_fonts;
-
   /* keeps track of the system font aliases that we might have */
   GHashTable *aliases;
 
-  double resolution;		/* (points / pixel) * PANGO_SCALE */
+  /* keeps track of the warned fonts that we might have */
+  GHashTable *warned_fonts;
 
-  /* IDWriteFontSetBuilder for loading custom fonts on Windows 10+ */
-  PangoWin32DWriteFontSetBuilder *font_set_builder;
+  double resolution;		/* (points / pixel) * PANGO_SCALE */
 };
 
 struct _PangoWin32FontMapClass
@@ -133,7 +122,6 @@ struct _PangoWin32Font
 
   LOGFONTW logfontw;
   int size;
-  char *variations;
 
   GSList *metrics_by_lang;
 
@@ -149,9 +137,6 @@ struct _PangoWin32Font
    */
   gboolean in_cache;
   GHashTable *glyph_info;
-
-  /* whether the font supports hinting */
-  gboolean is_hinted;
 };
 
 struct _PangoWin32FontClass
@@ -278,7 +263,7 @@ void            _pango_win32_make_matching_logfontw (PangoFontMap   *fontmap,
 						     LOGFONTW       *out);
 
 _PANGO_EXTERN
-GType           pango_win32_font_map_get_type      (void) G_GNUC_CONST;
+GType           _pango_win32_font_map_get_type      (void) G_GNUC_CONST;
 
 _PANGO_EXTERN
 void            _pango_win32_fontmap_cache_remove   (PangoFontMap   *fontmap,
@@ -294,57 +279,6 @@ _PANGO_EXTERN
 gpointer        _pango_win32_copy_cmap (gpointer cmap,
                                         guint16 cmap_format);
 
-_PANGO_EXTERN
-gboolean        pango_win32_dwrite_font_check_is_hinted       (PangoWin32Font    *font);
-
-_PANGO_EXTERN
-void           *pango_win32_font_get_dwrite_font_face         (PangoWin32Font    *font);
-
 extern gboolean _pango_win32_debug;
 
-void                   pango_win32_insert_font                (PangoWin32FontMap     *win32fontmap,
-                                                               LOGFONTW              *lfp,
-                                                               gpointer               dwrite_font,
-                                                               gboolean               is_synthetic);
-
-PangoWin32DWriteItems *pango_win32_init_direct_write          (void);
-
-PangoWin32DWriteItems *pango_win32_get_direct_write_items     (void);
-
-void                   pango_win32_dwrite_font_map_populate   (PangoWin32FontMap     *map);
-
-void                   pango_win32_dwrite_items_destroy       (PangoWin32DWriteItems *items);
-
-gboolean               pango_win32_dwrite_font_is_monospace   (gpointer               dwrite_font,
-                                                               gboolean              *is_monospace);
-
-void                   pango_win32_dwrite_font_release        (gpointer               dwrite_font);
-
-_PANGO_EXTERN
-void                   pango_win32_dwrite_font_face_release   (gpointer               dwrite_font_face);
-
-gpointer               pango_win32_logfontw_get_dwrite_font   (LOGFONTW              *logfontw);
-
-PangoFontDescription *
-pango_win32_font_description_from_logfontw_dwrite             (const LOGFONTW        *logfontw);
-
-hb_face_t *
-pango_win32_font_create_hb_face_dwrite                        (PangoWin32Font        *font);
-
-PangoFontDescription *
-pango_win32_font_description_from_dwrite_font                 (void                  *dwrite_font);
-
-/* This needs to be called from PangoCairo as well */
-_PANGO_EXTERN
-void                  pango_win32_font_map_cache_clear        (PangoFontMap          *font_map);
-
-gboolean              pango_win32_dwrite_add_font_file        (PangoFontMap          *font_map,
-                                                               const char            *font_file_path,
-                                                               GError               **error);
-
-void
-pango_win32_dwrite_release_font_set_builders                  (PangoWin32FontMap     *win32fontmap);
-
-G_END_DECLS
-
 #endif /* __PANGOWIN32_PRIVATE_H__ */
