@@ -19,6 +19,9 @@ void print_attributes (GSList        *attrs,
 void print_attr_list (PangoAttrList  *attrs,
                       GString        *string);
 
+PangoAttribute *
+attribute_from_string (const char *string);
+
 PangoAttrList *
 attributes_from_string (const char *string);
 
